#######################################################################
### keycodes67.py
###
### Usage: python keycodes67.py [input file [output file] ]
###
### If an input file is not specified, input is from standard input.
### If an output file is not specified, output is standard output.
###
### Synopsis:
### Read a HP-67/97 program text file and annotate with keycodes. All
### lines that are not program lines are output unchanged. The
### text file output is generated by the Teenix MultiCalc emulator's
### program editor.
###
### Detail:
### A program line is expected to start with a line number followed by
### a period and space, then the key name, such as 'GTO 5' or '+'.
### When a program line is recognized, a comment will be appended to
### the line with the keycode equivalent for the key.
###
### The key code is what is seen in the calculator display while in
### program mode, and the key code can be used to verify that a program
### has been entered correctly. Annotated program lines will appear
### like the ones shown below:
###
### 23. h ABS        //  35 64
### 24. RCL E        //  34 15
### 25. g x>y?       //  32 81
### 26. GTO 5        //  22 05
###
#######################################################################

import re
import sys
import os

# Global constants
keynames67 = [
    '0', '1', '2', '3', '4', '5', '6', '7',
    '8', '9', 'Sig+', 'GTO 0', 'GTO 1', 'GTO 2', 'GTO 3', 'GTO 4',
    'GTO 5', 'GTO 6', 'GTO 7', 'GTO 8', 'GTO 9', 'GTO A', 'GTO B', 'GTO C',
    'GTO D', 'GTO E', 'GTO (i)', 'GTO a', 'GTO b', 'GTO c', 'GTO d', 'GTO e',
    'DSP 0', 'DSP 1', 'DSP 2', 'DSP 3', 'DSP 4', 'DSP 5', 'DSP 6', 'DSP 7',
    'DSP 8', 'DSP 9', 'DSP (i)', 'f X Bar', 'f GSB 0', 'f GSB 1', 'f GSB 2', 'f GSB 3',
    'f GSB 4', 'f GSB 5', 'f GSB 6', 'f GSB 7', 'f GSB 8', 'f GSB 9', 'f GSB A', 'f GSB B',
    'f GSB C', 'f GSB D', 'f GSB E', 'f GSB (i)', 'f FIX', 'f RND', 'f LBL 0', 'f LBL 1',
    'f LBL 2', 'f LBL 3', 'f LBL 4', 'f LBL 5', 'f LBL 6', 'f LBL 7', 'f LBL 8', 'f LBL 9',
    'f LBL A', 'f LBL B', 'f LBL C', 'f LBL D', 'f LBL E', 'f LBL (i)', 'f DSZ', 'f ISZ',
    'f WDATA', 'f P<->S', 'f CLREG', 'f x=0?', 'f ln', 'f LOG', 'f sqrtX', 'f x!=0?',
    'f SIN', 'f COS', 'f TAN', 'f x<0?', 'f P->R', 'f R->D', 'f HMS->H', 'f x>0?',
    'f %', 'f INT', 'f -x-', 'g s', 'f GSB a', 'f GSB b', 'f GSB c', 'f GSB d',
    'f GSB e', 'g SCI', 'f LBL a', 'f LBL b', 'f LBL c', 'f LBL d', 'f LBL e', 'g DSZ (i)',
    'g ISZ (i)', 'g MERGE', 'g x=y?', 'g ex', 'g 10toX', 'g Xsqr', 'g x!=y?', 'g SIN-1',
    'g COS-1', 'g TAN-1', 'g x<=y?', 'g R->P', 'g D->R', 'g H->HMS', 'g x>y?', 'g % CH',
    'g FRAC', 'g STK', 'STO 0', 'STO 1', 'STO 2', 'STO 3', 'STO 4', 'STO 5',
    'STO 6', 'STO 7', 'STO 8', 'STO 9', 'STO A', 'STO B', 'STO C', 'STO D',
    'STO E', 'h STO (i)', 'STO - 0', 'STO - 1', 'STO - 2', 'STO - 3', 'STO - 4', 'STO - 5',
    'STO - 6', 'STO - 7', 'STO - 8', 'STO - 9', 'STO - (i)', 'STO + 0', 'STO + 1', 'STO + 2',
    'STO + 3', 'STO + 4', 'STO + 5', 'STO + 6', 'STO + 7', 'STO + 8', 'STO + 9', 'STO + (i)',
    'STO x 0', 'STO x 1', 'STO x 2', 'STO x 3', 'STO x 4', 'STO x 5', 'STO x 6', 'STO x 7',
    'STO x 8', 'STO x 9', 'STO x (i)', 'STO / 0', 'STO / 1', 'STO / 2', 'STO / 3', 'STO / 4',
    'STO / 5', 'STO / 6', 'STO / 7', 'STO / 8', 'STO / 9', 'STO / (i)', 'RCL 0', 'RCL 1',
    'RCL 2', 'RCL 3', 'RCL 4', 'RCL 5', 'RCL 6', 'RCL 7', 'RCL 8', 'RCL 9',
    'RCL A', 'RCL B', 'RCL C', 'RCL D', 'RCL E', 'h RCL Sig+', 'h RCL (i)', 'h Sig-',
    'h RTN', 'h ENG', 'h x<->i', 'h STO I', 'h RCL I', 'h DEG', 'h RAD', 'h GRAD',
    'h SF 0', 'h SF 1', 'h SF 2', 'h SF 3', 'h X<->Y', 'h ROLDN', 'h ROLUP', 'h CF 0',
    'h CF 1', 'h CF 2', 'h CF 3', 'h 1/x', 'h YtoX', 'h ABS', 'h F0set?', 'h F1set?',
    'h F2set?', 'h F3set?', 'h PAUSE', 'h pi', 'h REG', 'h n!', 'h LASTx', 'h HMS+',
    'h SPACE', 'ENTER', 'CHS', 'EEX', 'CLx', '-', '+', 'X',
    '/', '.', 'R/S'
]

keycodes67 = [
    '00', '01', '02', '03', '04', '05', '06', '07', 
    '08', '09', '21', '22 00', '22 01', '22 02', '22 03', '22 04', 
    '22 05', '22 06', '22 07', '22 08', '22 09', '22 11', '22 12', '22 13', 
    '22 14', '22 15', '22 24', '22 31 11', '22 31 12', '22 31 13', '22 31 14', '22 31 15', 
    '23 00', '23 01', '23 02', '23 03', '23 04', '23 05', '23 06', '23 07', 
    '23 08', '23 09', '23 24', '31 21', '31 22 00', '31 22 01', '31 22 02', '31 22 03', 
    '31 22 04', '31 22 05', '31 22 06', '31 22 07', '31 22 08', '31 22 09', '31 22 11', '31 22 12', 
    '31 22 13', '31 22 14', '31 22 15', '31 22 24', '31 23', '31 24', '31 25 00', '31 25 01', 
    '31 25 02', '31 25 03', '31 25 04', '31 25 05', '31 25 06', '31 25 07', '31 25 08', '31 25 09', 
    '31 25 11', '31 25 12', '31 25 13', '31 25 14', '31 25 15', '31 25 24', '31 33', '31 34', 
    '31 41', '31 42', '31 43', '31 51', '31 52', '31 53', '31 54', '31 61', 
    '31 62', '31 63', '31 64', '31 71', '31 72', '31 73', '31 74', '31 81', 
    '31 82', '31 83', '31 84', '32 21', '32 22 11', '32 22 12', '32 22 13', '32 22 14', 
    '32 22 15', '32 23', '32 25 11', '32 25 12', '32 25 13', '32 25 14', '32 25 15', '32 33', 
    '32 34', '32 41', '32 51', '32 52', '32 53', '32 54', '32 61', '32 62', 
    '32 63', '32 64', '32 71', '32 72', '32 73', '32 74', '32 81', '32 82', 
    '32 83', '32 84', '33 00', '33 01', '33 02', '33 03', '33 04', '33 05', 
    '33 06', '33 07', '33 08', '33 09', '33 11', '33 12', '33 13', '33 14', 
    '33 15', '33 24', '33 51 00', '33 51 01', '33 51 02', '33 51 03', '33 51 04', '33 51 05', 
    '33 51 06', '33 51 07', '33 51 08', '33 51 09', '33 51 24', '33 61 00', '33 61 01', '33 61 02', 
    '33 61 03', '33 61 04', '33 61 05', '33 61 06', '33 61 07', '33 61 08', '33 61 09', '33 61 24', 
    '33 71 00', '33 71 01', '33 71 02', '33 71 03', '33 71 04', '33 71 05', '33 71 06', '33 71 07', 
    '33 71 08', '33 71 09', '33 71 24', '33 81 00', '33 81 01', '33 81 02', '33 81 03', '33 81 04', 
    '33 81 05', '33 81 06', '33 81 07', '33 81 08', '33 81 09', '33 81 24', '34 00', '34 01', 
    '34 02', '34 03', '34 04', '34 05', '34 06', '34 07', '34 08', '34 09', 
    '34 11', '34 12', '34 13', '34 14', '34 15', '34 21', '34 24', '35 21', 
    '35 22', '35 23', '35 24', '35 33', '35 34', '35 41', '35 42', '35 43', 
    '35 51 00', '35 51 01', '35 51 02', '35 51 03', '35 52', '35 53', '35 54', '35 61 00', 
    '35 61 01', '35 61 02', '35 61 03', '35 62', '35 63', '35 64', '35 71 00', '35 71 01', 
    '35 71 02', '35 71 03', '35 72', '35 73', '35 74', '35 81', '35 82', '35 83', 
    '35 84', '41', '42', '43', '44', '51', '61', '71', 
    '81', '83', '84', 
]

# Global variables
regexp = re.compile(r"(?P<lineno>[0-9]+). " r"(?P<name>.+$)")
inf = sys.stdin
outf = sys.stdout
comstart=17

# Retrieve the one, two or three digit pairs that represent a key in program memory
# Order N squared (so sue me)
def getKeycode(name):
    if name in keynames67:
        return keycodes67[keynames67.index(name)]
    else:
        return 'N/A'

# Output a line unaltered if not in proper program format or append keycode comment
# Pad the line out with spaces preceeding the comment
def processLine(line):
    result = regexp.search(line)
    if result == None:
        outf.write(line+'\n')
    else:
        name = result.group('name')
        pad = " " * (comstart-len(line))
        outf.write( line+pad+'//  '+getKeycode(name)+'\n' )

def parseArgs():
    global inf, outf

    if len(sys.argv) > 1:
        inf = open(sys.argv[1], 'r')
    if len(sys.argv) > 2:
        outf = open(sys.argv[2], 'w')


def main():
    parseArgs()
    lines = inf.read().split("\n")
    for line in lines:
        processLine(line)


if __name__ == '__main__':
    main()
    